import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../utils/classnames';

import './index.css';

// 不用 react-weui 而直接用底层实现的原因是 react-weui 不能支持表单类型

export default function Button({
  text = '按钮',
  size = 'default',
  type = 'primary',
  contentSlot,
  formType,
  loading = false,
  disabled = false,
  events = {},
  className,
  ...props
}) {
  const cls = classNames({
    'weda-ui': true,
    'weda-button': true,
    'weui-btn': true,
    'weui-btn_mini': size === 'mini',
    'weui-btn_primary': type === 'primary',
    'weui-btn_default': type === 'default',
    'weui-btn_warn': type === 'warn',
    'weui-btn_disabled': disabled,
    [className]: className,
  });

  return (
    <button
      className={cls}
      disabled={disabled}
      type={formType}
      {...props}
      onClick={(e) => events.tap({}, { originEvent: e })}
    >
      {loading && (
        <span className="weui-btn_loading">
          <i className="weui-loading" />
        </span>
      )}
      {text ? text : contentSlot}
    </button>
  );
}

Button.propTypes = {
  /**
   * 内容
   */
  text: PropTypes.string,

  /**
   * 样式类型
   */
  type: PropTypes.oneOf(['primary', 'warn', 'default']),

  /**
   * 大小
   */
  size: PropTypes.oneOf(['default', 'mini']),

  /**
   * 是否禁用
   */
  disabled: PropTypes.bool,

  /**
   * 加载中
   */
  loading: PropTypes.bool,

  /**
   * 组件事件
   */
  events: PropTypes.objectOf(PropTypes.func),

  /**
   * 用于 form 类型
   */
  formType: PropTypes.oneOf(['button', 'submit', 'reset']),

  className: PropTypes.string,
  contentSlot: PropTypes.string,
};

/**
 * 默认属性
 */
Button.defaultProps = {
  type: 'primary',
  size: 'default',
};
