import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../../utils/classnames';
import getLocalCounter from '../../../utils/getLocalCounter';
import isObjectEqual from '../../../utils/isObjectEqual';
import FormCell from '../formcell';
import fromEntries from 'object.fromentries';

export default function Checkbox({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  name,
  label,
  labelVisible = true,
  range,
  layout,
  disabled,
  requiredFlag,
}) {
  const cls = classNames({
    'weda-ui': true,
    'weda-checkbox': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-check__label': true,
    'weui-cell_disabled': disabled,
  });

  const [checkedItems, setCheckedItems] = React.useState(
    fromEntries(range.map(({ value, checked }) => [value, !!checked]))
  );
  // 两次range不同时, 需要刷新checkedItems
  const prevRangeRef = React.useRef();
  React.useEffect(() => {
    if (!isObjectEqual(prevRangeRef.current, range)) {
      prevRangeRef.current = range;
      setCheckedItems(
        fromEntries(range.map(({ value, checked }) => [value, !!checked]))
      );
    }
  });

  const onChange = function (e) {
    const { value, checked } = e.target;
    const newMap = Object.assign({}, checkedItems, { [value]: checked });
    setCheckedItems(newMap);
    events.change(
      {
        value: Object.entries(newMap)
          .filter(([value, checked]) => checked)
          .map(([value, checked]) => value),
      },
      { originEvent: e }
    );
  };

  // 现在低码组件往源码组件传入的id不唯一, 暂时使用hashcode替代
  const hashcode = getLocalCounter();

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={true}
      requiredFlag={requiredFlag}
    >
      {range.map(({ label, value, checked }, index) => {
        const _id = `checkbox-${hashcode}-${index}`;
        const _name = `checkbox-${hashcode}-${name}`;
        return (
          <label className={subCls} htmlFor={_id} key={index}>
            <div className="weui-cell__hd">
              <input
                type="checkbox"
                className="weui-check"
                value={value}
                name={_name}
                id={_id}
                checked={checkedItems[value]}
                disabled={disabled}
                onChange={onChange}
              />
              <i className="weui-icon-checked"></i>
            </div>
            <div className="weui-cell__bd">
              <p>{label}</p>
            </div>
          </label>
        );
      })}
    </FormCell>
  );
}

Checkbox.propTypes = {
  // 系统属性
  id: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.string,
  events: PropTypes.objectOf(PropTypes.func),
  // 组件属性
  name: PropTypes.string,
  label: PropTypes.string,
  labelVisible: PropTypes.bool,
  range: PropTypes.array,
  layout: PropTypes.string,
  disabled: PropTypes.bool,
  requiredFlag: PropTypes.bool,
};

Checkbox.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  name: 'formCheckbox',
  label: '标题',
  labelVisible: true,
  range: [],
  layout: 'horizontal',
  disabled: false,
  requiredFlag: false,
};
