import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../../utils/classnames';
import getLocalCounter from '../../../utils/getLocalCounter';
import isObjectEqual from '../../../utils/isObjectEqual';
import FormCell from '../formcell';

export default function Radio({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  label,
  labelVisible,
  name,
  range,
  layout,
  disabled,
  requiredFlag,
}) {
  const cls = classNames({
    'weda-ui': true,
    'weda-radio': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-check__label': true,
    'weui-cell_disabled': disabled,
  });

  const [checkedItemValue, setCheckedItemValue] = React.useState(
    range.find((item) => item.checked)?.value
  );
  // 两次range不同时, 需要刷新checkedItemValue
  const prevRangeRef = React.useRef();
  React.useEffect(() => {
    if (!isObjectEqual(prevRangeRef.current, range)) {
      prevRangeRef.current = range;
      setCheckedItemValue(range.find((item) => item.checked)?.value);
    }
  });

  const onChange = function (e) {
    const value = e.target.value;
    setCheckedItemValue(value);
    events.change({ value }, { originEvent: e });
  };

  // 现在低码组件往源码组件传入的id不唯一, 暂时使用hashcode替代
  const hashcode = getLocalCounter();

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={true}
      requiredFlag={requiredFlag}
    >
      {range.map(({ label, value }, index) => {
        const _id = `radio-${hashcode}-${index}`;
        const _name = `radio-${hashcode}-${name}`;
        return (
          <label className={subCls} htmlFor={_id} key={_id}>
            <div className="weui-cell__hd">
              <input
                type="radio"
                className="weui-check"
                value={value}
                name={_name}
                id={_id}
                checked={checkedItemValue === value}
                disabled={disabled}
                onChange={onChange}
              />
              <i className="weui-icon-checked"></i>
            </div>
            <div className="weui-cell__bd">
              <p>{label}</p>
            </div>
          </label>
        );
      })}
    </FormCell>
  );
}

Radio.propTypes = {
  // 系统属性
  id: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.string,
  events: PropTypes.objectOf(PropTypes.func),
  // 组件属性
  label: PropTypes.string,
  labelVisible: PropTypes.bool,
  name: PropTypes.string,
  range: PropTypes.array,
  layout: PropTypes.string,
  disabled: PropTypes.bool,
  requiredFlag: PropTypes.bool,
};

Radio.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  name: 'formRadio',
  range: [],
  layout: 'horizontal',
  disabled: false,
  requiredFlag: false,
};
