import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../../utils/classnames';
import FormCell from '../formcell';

export default function Switch({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  label,
  labelVisible,
  checked: defaultChecked,
  layout,
  disabled,
  requiredFlag,
}) {
  const isFlex = layout !== 'vertical';
  const cls = classNames({
    'weda-ui': true,
    'weda-switch': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-cell_switch': true,
    'weui-flex__item': isFlex,
    'weui-cell_disabled': disabled,
  });

  const [checked, setChecked] = React.useState(defaultChecked);
  // 两次默认值不同时, 需要刷新
  const prevDefaultCheckedRef = React.useRef();
  React.useEffect(() => {
    if (prevDefaultCheckedRef.current !== defaultChecked) {
      prevDefaultCheckedRef.current = defaultChecked;
      setChecked(defaultChecked);
    }
  });

  const onChange = function (e) {
    setChecked(e.target.checked);
    events.change({ value: e.target.checked }, { originEvent: e });
  };

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={true}
      requiredFlag={requiredFlag}
    >
      <div className={subCls}>
        {isFlex ? <div className="weui-cell__bd"></div> : null}
        <div className={isFlex ? 'weui-cell__ft' : 'weui-cell__bd'}>
          <label htmlFor={`switch-${id}`} className="weui-switch-cp">
            <input
              id={`switch-${id}`}
              className="weui-switch-cp__input"
              type="checkbox"
              checked={checked}
              disabled={disabled}
              onChange={onChange}
            />
            <div className="weui-switch-cp__box"></div>
          </label>
        </div>
      </div>
    </FormCell>
  );
}

Switch.propTypes = {
  // 系统属性
  id: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.string,
  events: PropTypes.objectOf(PropTypes.func),
  // 组件属性
  label: PropTypes.string,
  labelVisible: PropTypes.boolean,
  disabled: PropTypes.bool,
  checked: PropTypes.bool,
  layout: PropTypes.string,
  requiredFlag: PropTypes.bool,
};

Switch.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  disabled: false,
  checked: false,
  layout: PropTypes.string,
  requiredFlag: false,
};
