import * as React from 'react';
import * as PropTypes from 'prop-types';

import classNames from '../../../utils/classnames';
import FormCell from '../formcell';
import './index.css';

export default function Textarea({
  // 系统属性
  id,
  className,
  style,
  events,
  // 组件属性
  label,
  labelVisible,
  name,
  defaultValue,
  placeholder,
  maxLength,
  disabled,
  focus: autoFocus,
  layout,
  requiredFlag,
  counterVisible,
}) {
  console.log('weda-textarea', defaultValue);
  const cls = classNames({
    'weda-ui': true,
    'weda-textarea': true,
    [className]: className,
  });
  const subCls = classNames({
    'weui-cell': true,
    'weui-cell_active': true,
    'weui-cell_form': true,
    'weui-cell_disabled': disabled,
  });

  const [value, setValue] = React.useState(defaultValue);
  const [counter, setCounter] = React.useState(defaultValue?.length || 0);
  // 两次默认值不同时, 需要刷新value
  const prevDefaultValueRef = React.useRef();
  React.useEffect(() => {
    if (prevDefaultValueRef.current !== defaultValue) {
      prevDefaultValueRef.current = defaultValue;
      setValue(defaultValue);
      setCounter(defaultValue?.length || 0);
    }
  });

  const onChange = function (e) {
    setValue(e?.target?.value);
    setCounter(e?.target?.value?.length || 0);
    events.change({ value: e?.target?.value });
  };

  return (
    <FormCell
      className={cls}
      style={style}
      label={labelVisible ? label : null}
      layout={layout}
      multiCell={false}
      requiredFlag={requiredFlag}
    >
      <div className={subCls}>
        <div className="weui-cell__bd">
          <textarea
            className="weui-textarea"
            placeholder={placeholder}
            value={value}
            onChange={onChange}
            maxLength={maxLength}
            disabled={disabled}
            autoFocus={autoFocus}
            style={style}
          ></textarea>
          {counterVisible ? (
            <div className="weui-textarea-counter">
              <span>{counter}</span>/{maxLength}
            </div>
          ) : null}
        </div>
      </div>
    </FormCell>
  );
}

Textarea.propTypes = {
  // 系统属性
  id: PropTypes.string,
  style: PropTypes.string,
  className: PropTypes.string,
  events: PropTypes.object,
  // 组件属性
  label: PropTypes.string,
  labelVisible: PropTypes.bool,
  name: PropTypes.string,
  defaultValue: PropTypes.string,
  placeholder: PropTypes.string,
  maxLength: PropTypes.number,
  disabled: PropTypes.bool,
  focus: PropTypes.bool,
  layout: PropTypes.oneOf(['horizontal', 'vertical']),
  requiredFlag: PropTypes.bool,
  counterVisible: PropTypes.bool,
};

Textarea.defaultProps = {
  // 系统属性
  events: {},
  // 组件属性
  label: '标题',
  labelVisible: true,
  nane: 'formTextarea',
  defaultValue: '',
  placeholder: '请输入',
  maxLength: 140,
  disabled: false,
  focus: false,
  layout: 'horizontal',
  requiredFlag: false,
  counterVisible: true,
};
