import * as React from 'react';
import weui from '../../../utils/weui';

import classNames from '../../../utils/classnames';
import { getCloudInstance } from '../../../utils/tcb';

/**
 * H5端
 */
export function UploaderH5({
  maxUploadCount,
  className,
  id,
  events,
  ...props
}) {
  const cls = classNames({
    'weda-ui': true,
    'weui-cells': true,
    'weui-cells_form': true,
    [className]: className,
  });

  const [uploadCount, setUploadConut] = React.useState(0);

  React.useEffect(() => {
    const uploader = weui.uploader(`#${id}`, {
      url: 'http://localhost:8081',
      auto: true,
      type: 'file',
      fileVal: 'fileVal',
      compress: {
        width: 1600,
        height: 1600,
        quality: 0.8,
      },
      onBeforeQueued: function (files) {
        // `this` 是轮询到的文件, `files` 是所有文件

        if (
          ['image/jpg', 'image/jpeg', 'image/png', 'image/gif'].indexOf(
            this.type
          ) < 0
        ) {
          weui.alert('请上传图片');
          return false; // 阻止文件添加
        }
        if (this.size > 10 * 1024 * 1024) {
          weui.alert('请上传不超过10M的图片');
          return false;
        }
        if (files.length > maxUploadCount) {
          // 防止一下子选择过多文件
          weui.alert(`最多只能上传${maxUploadCount}张图片，请重新选择`);
          return false;
        }
        if (uploadCount + 1 > maxUploadCount) {
          weui.alert(`最多只能上传${maxUploadCount}张图片`);
          return false;
        }

        setUploadConut(uploadCount + 1);

        // return true; // 阻止默认行为，不插入预览图的框架
      },
      onQueued: function () {
        // console.log(this);

        // console.log(this.status); // 文件的状态：'ready', 'progress', 'success', 'fail'
        // console.log(this.base64); // 如果是base64上传，file.base64可以获得文件的base64

        // this.upload(); // 如果是手动上传，这里可以通过调用upload来实现；也可以用它来实现重传。
        // this.stop(); // 中断上传

        // return true; // 阻止默认行为，不显示预览图的图像
        const file = this;
        getCloudInstance().then(async (tcb) => {
          try {
            const fileType = file.type.split('/')?.[1];
            const cloudPath = `temp/uploader-${Date.now()}.${fileType}`;
            const { fileID } = await tcb.uploadFile({
              cloudPath,
              filePath: file,
            });
            const { fileList } = await tcb.getTempFileURL({
              fileList: [fileID],
            });
            uploader.onSuccess(this, [
              {
                download_url: fileList?.[0]?.download_url,
                fileID,
              },
            ]);
          } catch (err) {
            uploader.onError(this, err);
          }
        });
      },
      onBeforeSend: function (data, headers) {
        // $.extend(data, { test: 1 }); // 可以扩展此对象来控制上传参数
        // $.extend(headers, { Origin: 'http://127.0.0.1' }); // 可以扩展此对象来控制上传头部

        return false; // 阻止文件上传
      },
      onProgress: function (percent) {
        // console.log(this, percent);
        // return true; // 阻止默认行为，不使用默认的进度显示
      },
      onSuccess: function (ret) {
        // console.log(this, ret);
        // return true; // 阻止默认行为，不使用默认的成功态
        events.success(ret);
      },
      onError: function (err) {
        // console.log(this, err);
        // return true; // 阻止默认行为，不使用默认的失败态
        events.error(err);
      },
    });
  }, []);

  return (
    <div className={cls} id={id} {...props}>
      <div className="weui-cell">
        <div className="weui-cell__bd">
          <div className="weui-uploader">
            <div className="weui-uploader__hd">
              <p className="weui-uploader__title">图片上传</p>
              <div className="weui-uploader__info">
                <span id="uploadCount">{uploadCount}</span>
                {`/${maxUploadCount}`}
              </div>
            </div>
            <div className="weui-uploader__bd">
              <ul className="weui-uploader__files" id="uploaderFiles"></ul>
              <div className="weui-uploader__input-box">
                <input
                  id="uploaderInput"
                  className="weui-uploader__input"
                  type="file"
                  accept="image/*"
                  capture="camera"
                  multiple=""
                />
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  );
}
