import * as React from 'react';
import classNames from '../../utils/classnames';
import './index.css';
import * as PropTypes from 'prop-types';

export default function Link({
  content = '这是一个链接',
  url,
  events = {},
  style,
  className,
  isBlank = false,
  pageId = 'index',
  type,
  params = [],
  preventDefault,
  navigator = 'navigateTo',
}) {
  const cls = classNames({
    'weda-link': true,
    [className]: className,
  });

  let paramStr = params.length
    ? params.reduce((total, cur, index) => {
        return (
          total +
          cur.key +
          '=' +
          cur.value +
          (index != params.length - 1 ? '&' : '')
        );
      }, '?')
    : '';
  let paramObj = params.reduce((total, cur) => {
    return Object.assign({}, total, { [cur.key]: cur.value });
  }, {});

  return (
    <a
      style={{ ...style }}
      className={cls}
      href="#!"
      onClick={(e) => {
        e.preventDefault();
        if (!preventDefault) {
          if (isBlank) {
            window.open('https://' + url + paramStr);
          } else {
            if (type === 'url')
              window.location.href = 'https://' + url + paramStr;
            else
              app[navigator]({
                pageId: pageId,
                packageName: '',
                mode: 'weDa',
                params: paramObj,
              });
          }
        }
        events.tap && events.tap({}, { originEvent: e });
      }}
    >
      {content}
    </a>
  );
}

Link.propTypes = {
  // 系统属性
  /**
   * 类名
   */
  className: PropTypes.string,
  /**
   * 事件
   */
  events: PropTypes.objectOf(PropTypes.func),
  /**
   * 内联样式
   */
  style: PropTypes.object,
  // 组件属性
  /**
   * 内容
   */
  content: PropTypes.string,
  /**
   * 链接地址
   */
  url: PropTypes.string,
  /**
   * 链接类型
   */
  type: PropTypes.oneOf(['page', 'url']),
  /**
   * 参数
   */
  params: PropTypes.array,
  /**
   * 新窗口打开
   */
  isBlank: PropTypes.bool,
  /**
   * 阻止默认行为
   */
  preventDefault: PropTypes.bool,
  /**
   * 路由方式
   */
  navigator: PropTypes.string,
  /**
   * 选择页面
   */
  pageId: PropTypes.string,
};
